@@ -3,24 +3,23 @@ package forward
 import (
 	"bufio"
 	"crypto/tls"
+	"errors"
 	"fmt"
 	"net"
 	"net/http"
 	"net/http/httptest"
-	"runtime"
 	"testing"
-	"time"
 
 	gorillawebsocket "github.com/gorilla/websocket"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
-	"github.com/vulcand/oxy/testutils"
+	"github.com/vulcand/oxy/v2/internal/holsterv4/clock"
+	"github.com/vulcand/oxy/v2/testutils"
 	"golang.org/x/net/websocket"
 )
 
 func TestWebSocketTCPClose(t *testing.T) {
-	f, err := New(PassHostHeader(true))
-	require.NoError(t, err)
+	f := New(true)
 
 	errChan := make(chan error, 1)
 	upgrader := gorillawebsocket.Upgrader{}
@@ -29,7 +28,7 @@ func TestWebSocketTCPClose(t *testing.T) {
 		if err != nil {
 			return
 		}
-		defer c.Close()
+		defer func(c *gorillawebsocket.Conn) { _ = c.Close() }(c)
 		for {
 			_, _, err := c.ReadMessage()
 			if err != nil {
@@ -38,7 +37,7 @@ func TestWebSocketTCPClose(t *testing.T) {
 			}
 		}
 	}))
-	defer srv.Close()
+	t.Cleanup(srv.Close)
 
 	proxy := createProxyWithForwarder(f, srv.URL)
 
@@ -48,69 +47,20 @@ func TestWebSocketTCPClose(t *testing.T) {
 		withPath("/ws"),
 	).open()
 	require.NoError(t, err)
-	conn.Close()
+	_ = conn.Close()
 
 	serverErr := <-errChan
 
-	wsErr, ok := serverErr.(*gorillawebsocket.CloseError)
-	assert.Equal(t, true, ok)
+	wsErr := &gorillawebsocket.CloseError{}
+	assert.ErrorAs(t, serverErr, &wsErr)
 	assert.Equal(t, 1006, wsErr.Code)
 }
 
-func TestWebsocketConnectionClosedHook(t *testing.T) {
-	closed := make(chan struct{})
-	f, err := New(WebsocketConnectionClosedHook(func(req *http.Request, conn net.Conn) {
-		close(closed)
-	}))
-	require.NoError(t, err)
-
-	mux := http.NewServeMux()
-	mux.Handle("/ws", websocket.Handler(func(conn *websocket.Conn) {
-		msg := make([]byte, 4)
-		conn.Read(msg)
-		conn.Write(msg)
-		conn.Close()
-	}))
-
-	srv := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
-		mux.ServeHTTP(w, req)
-	})
-	defer srv.Close()
-
-	proxy := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
-		req.URL = testutils.ParseURI(srv.URL)
-		f.ServeHTTP(w, req)
-	})
-	defer proxy.Close()
-
-	serverAddr := proxy.Listener.Addr().String()
-
-	headers := http.Header{}
-	webSocketURL := "ws://" + serverAddr + "/ws"
-	headers.Add("Origin", webSocketURL)
-
-	conn, resp, err := gorillawebsocket.DefaultDialer.Dial(webSocketURL, headers)
-	require.NoError(t, err, "Error during Dial with response: %+v", resp)
-
-	conn.WriteMessage(gorillawebsocket.TextMessage, []byte("OK"))
-	fmt.Println(conn.ReadMessage())
-
-	conn.Close()
-
-	select {
-	case <-time.After(time.Second):
-		t.Errorf("Websocket Hook not called")
-	case <-closed:
-
-	}
-}
-
 func TestWebSocketPingPong(t *testing.T) {
-	f, err := New()
-	require.NoError(t, err)
+	f := New(true)
 
-	var upgrader = gorillawebsocket.Upgrader{
-		HandshakeTimeout: 10 * time.Second,
+	upgrader := gorillawebsocket.Upgrader{
+		HandshakeTimeout: 10 * clock.Second,
 		CheckOrigin: func(*http.Request) bool {
 			return true
 		},
@@ -122,23 +72,21 @@ func TestWebSocketPingPong(t *testing.T) {
 		require.NoError(t, err)
 
 		ws.SetPingHandler(func(appData string) error {
-			ws.WriteMessage(gorillawebsocket.PongMessage, []byte(appData+"Pong"))
+			_ = ws.WriteMessage(gorillawebsocket.PongMessage, []byte(appData+"Pong"))
 			return nil
 		})
 
-		ws.ReadMessage()
+		_, _, _ = ws.ReadMessage()
 	})
 
-	srv := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
-		mux.ServeHTTP(w, req)
-	})
-	defer srv.Close()
+	srv := httptest.NewServer(mux)
+	t.Cleanup(srv.Close)
 
 	proxy := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
 		req.URL = testutils.ParseURI(srv.URL)
 		f.ServeHTTP(w, req)
 	})
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	serverAddr := proxy.Listener.Addr().String()
 
@@ -159,37 +107,34 @@ func TestWebSocketPingPong(t *testing.T) {
 		return badErr
 	})
 
-	conn.WriteControl(gorillawebsocket.PingMessage, []byte("Ping"), time.Now().Add(time.Second))
+	_ = conn.WriteControl(gorillawebsocket.PingMessage, []byte("Ping"), clock.Now().Add(clock.Second))
 	_, _, err = conn.ReadMessage()
 
-	if err != goodErr {
+	if !errors.Is(err, goodErr) {
 		require.NoError(t, err)
 	}
 }
 
 func TestWebSocketEcho(t *testing.T) {
-	f, err := New()
-	require.NoError(t, err)
+	f := New(true)
 
 	mux := http.NewServeMux()
 	mux.Handle("/ws", websocket.Handler(func(conn *websocket.Conn) {
 		msg := make([]byte, 4)
-		conn.Read(msg)
-		fmt.Println(string(msg))
-		conn.Write(msg)
-		conn.Close()
+		_, _ = conn.Read(msg)
+		t.Log(string(msg))
+		_, _ = conn.Write(msg)
+		_ = conn.Close()
 	}))
 
-	srv := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
-		mux.ServeHTTP(w, req)
-	})
-	defer srv.Close()
+	srv := httptest.NewServer(mux)
+	t.Cleanup(srv.Close)
 
 	proxy := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
 		req.URL = testutils.ParseURI(srv.URL)
 		f.ServeHTTP(w, req)
 	})
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	serverAddr := proxy.Listener.Addr().String()
 
@@ -200,10 +145,10 @@ func TestWebSocketEcho(t *testing.T) {
 	conn, resp, err := gorillawebsocket.DefaultDialer.Dial(webSocketURL, headers)
 	require.NoError(t, err, "Error during Dial with response: %+v", resp)
 
-	conn.WriteMessage(gorillawebsocket.TextMessage, []byte("OK"))
-	fmt.Println(conn.ReadMessage())
+	_ = conn.WriteMessage(gorillawebsocket.TextMessage, []byte("OK"))
+	t.Log(conn.ReadMessage())
 
-	conn.Close()
+	_ = conn.Close()
 }
 
 func TestWebSocketPassHost(t *testing.T) {
@@ -225,10 +170,7 @@ func TestWebSocketPassHost(t *testing.T) {
 
 	for _, test := range testCases {
 		t.Run(test.desc, func(t *testing.T) {
-			f, err := New()
-			f.passHost = test.passHost
-
-			require.NoError(t, err)
+			f := New(test.passHost)
 
 			mux := http.NewServeMux()
 			mux.Handle("/ws", websocket.Handler(func(conn *websocket.Conn) {
@@ -241,22 +183,20 @@ func TestWebSocketPassHost(t *testing.T) {
 				}
 
 				msg := make([]byte, 4)
-				conn.Read(msg)
-				fmt.Println(string(msg))
-				conn.Write(msg)
-				conn.Close()
+				_, _ = conn.Read(msg)
+				t.Log(string(msg))
+				_, _ = conn.Write(msg)
+				_ = conn.Close()
 			}))
 
-			srv := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
-				mux.ServeHTTP(w, req)
-			})
-			defer srv.Close()
+			srv := httptest.NewServer(mux)
+			t.Cleanup(srv.Close)
 
 			proxy := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
 				req.URL = testutils.ParseURI(srv.URL)
 				f.ServeHTTP(w, req)
 			})
-			defer proxy.Close()
+			t.Cleanup(proxy.Close)
 
 			serverAddr := proxy.Listener.Addr().String()
 
@@ -268,62 +208,16 @@ func TestWebSocketPassHost(t *testing.T) {
 			conn, resp, err := gorillawebsocket.DefaultDialer.Dial(webSocketURL, headers)
 			require.NoError(t, err, "Error during Dial with response: %+v", resp)
 
-			conn.WriteMessage(gorillawebsocket.TextMessage, []byte("OK"))
-			fmt.Println(conn.ReadMessage())
+			_ = conn.WriteMessage(gorillawebsocket.TextMessage, []byte("OK"))
+			t.Log(conn.ReadMessage())
 
-			conn.Close()
+			_ = conn.Close()
 		})
 	}
 }
 
-func TestWebSocketNumGoRoutine(t *testing.T) {
-	t.Skip("Flaky on goroutine")
-	f, err := New()
-	require.NoError(t, err)
-
-	mux := http.NewServeMux()
-	mux.Handle("/ws", websocket.Handler(func(conn *websocket.Conn) {
-		msg := make([]byte, 4)
-		conn.Read(msg)
-		fmt.Println(string(msg))
-		conn.Write(msg)
-		conn.Close()
-	}))
-
-	srv := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
-		mux.ServeHTTP(w, req)
-	})
-	defer srv.Close()
-
-	proxy := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
-		req.URL = testutils.ParseURI(srv.URL)
-		f.ServeHTTP(w, req)
-	})
-	defer proxy.Close()
-
-	serverAddr := proxy.Listener.Addr().String()
-
-	num := runtime.NumGoroutine()
-
-	headers := http.Header{}
-	webSocketURL := "ws://" + serverAddr + "/ws"
-	headers.Add("Origin", webSocketURL)
-
-	conn, resp, err := gorillawebsocket.DefaultDialer.Dial(webSocketURL, headers)
-	require.NoError(t, err, "Error during Dial with response: %+v", resp)
-
-	conn.WriteMessage(gorillawebsocket.TextMessage, []byte("OK"))
-	fmt.Println(conn.ReadMessage())
-
-	conn.Close()
-
-	time.Sleep(time.Second)
-	assert.Equal(t, num, runtime.NumGoroutine())
-}
-
 func TestWebSocketServerWithoutCheckOrigin(t *testing.T) {
-	f, err := New()
-	require.NoError(t, err)
+	f := New(true)
 
 	upgrader := gorillawebsocket.Upgrader{CheckOrigin: func(r *http.Request) bool {
 		return true
@@ -345,10 +239,10 @@ func TestWebSocketServerWithoutCheckOrigin(t *testing.T) {
 			}
 		}
 	}))
-	defer srv.Close()
+	t.Cleanup(srv.Close)
 
 	proxy := createProxyWithForwarder(f, srv.URL)
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	proxyAddr := proxy.Listener.Addr().String()
 	resp, err := newWebsocketRequest(
@@ -363,8 +257,7 @@ func TestWebSocketServerWithoutCheckOrigin(t *testing.T) {
 }
 
 func TestWebSocketRequestWithOrigin(t *testing.T) {
-	f, err := New(PassHostHeader(true))
-	require.NoError(t, err)
+	f := New(true)
 
 	upgrader := gorillawebsocket.Upgrader{}
 	srv := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
@@ -384,13 +277,13 @@ func TestWebSocketRequestWithOrigin(t *testing.T) {
 			}
 		}
 	}))
-	defer srv.Close()
+	t.Cleanup(srv.Close)
 
 	proxy := createProxyWithForwarder(f, srv.URL)
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	proxyAddr := proxy.Listener.Addr().String()
-	_, err = newWebsocketRequest(
+	_, err := newWebsocketRequest(
 		withServer(proxyAddr),
 		withPath("/ws"),
 		withData("echo"),
@@ -409,8 +302,7 @@ func TestWebSocketRequestWithOrigin(t *testing.T) {
 }
 
 func TestWebSocketRequestWithQueryParams(t *testing.T) {
-	f, err := New(PassHostHeader(true))
-	require.NoError(t, err)
+	f := New(true)
 
 	upgrader := gorillawebsocket.Upgrader{}
 	srv := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
@@ -431,10 +323,10 @@ func TestWebSocketRequestWithQueryParams(t *testing.T) {
 			}
 		}
 	}))
-	defer srv.Close()
+	t.Cleanup(srv.Close)
 
 	proxy := createProxyWithForwarder(f, srv.URL)
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	proxyAddr := proxy.Listener.Addr().String()
 
@@ -449,24 +341,21 @@ func TestWebSocketRequestWithQueryParams(t *testing.T) {
 }
 
 func TestWebSocketRequestWithHeadersInResponseWriter(t *testing.T) {
-	f, err := New()
-	require.NoError(t, err)
+	f := New(true)
 
 	mux := http.NewServeMux()
 	mux.Handle("/ws", websocket.Handler(func(conn *websocket.Conn) {
-		conn.Close()
+		_ = conn.Close()
 	}))
-	srv := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
-		mux.ServeHTTP(w, req)
-	})
-	defer srv.Close()
+	srv := httptest.NewServer(mux)
+	t.Cleanup(srv.Close)
 
 	proxy := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
 		req.URL = testutils.ParseURI(srv.URL)
 		w.Header().Set("HEADER-KEY", "HEADER-VALUE")
 		f.ServeHTTP(w, req)
 	})
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	serverAddr := proxy.Listener.Addr().String()
 
@@ -475,14 +364,13 @@ func TestWebSocketRequestWithHeadersInResponseWriter(t *testing.T) {
 	headers.Add("Origin", webSocketURL)
 	conn, resp, err := gorillawebsocket.DefaultDialer.Dial(webSocketURL, headers)
 	require.NoError(t, err, "Error during Dial with response: %+v", err, resp)
-	defer conn.Close()
+	t.Cleanup(func() { _ = conn.Close() })
 
 	assert.Equal(t, "HEADER-VALUE", resp.Header.Get("HEADER-KEY"))
 }
 
 func TestWebSocketRequestWithEncodedChar(t *testing.T) {
-	f, err := New(PassHostHeader(true))
-	require.NoError(t, err)
+	f := New(true)
 
 	upgrader := gorillawebsocket.Upgrader{}
 	srv := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
@@ -503,10 +391,10 @@ func TestWebSocketRequestWithEncodedChar(t *testing.T) {
 			}
 		}
 	}))
-	defer srv.Close()
+	t.Cleanup(srv.Close)
 
 	proxy := createProxyWithForwarder(f, srv.URL)
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	proxyAddr := proxy.Listener.Addr().String()
 
@@ -520,43 +408,16 @@ func TestWebSocketRequestWithEncodedChar(t *testing.T) {
 	assert.Equal(t, "ok", resp)
 }
 
-func TestDetectsWebSocketRequest(t *testing.T) {
-	mux := http.NewServeMux()
-	mux.Handle("/ws", websocket.Handler(func(conn *websocket.Conn) {
-		conn.Write([]byte("ok"))
-		conn.Close()
-	}))
-	srv := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
-		websocketRequest := IsWebsocketRequest(req)
-		assert.Equal(t, true, websocketRequest)
-		mux.ServeHTTP(w, req)
-	})
-	defer srv.Close()
-
-	serverAddr := srv.Listener.Addr().String()
-
-	resp, err := newWebsocketRequest(
-		withServer(serverAddr),
-		withPath("/ws"),
-		withData("echo"),
-	).send()
-
-	require.NoError(t, err)
-	assert.Equal(t, "ok", resp)
-}
-
 func TestWebSocketUpgradeFailed(t *testing.T) {
-	f, err := New()
-	require.NoError(t, err)
+	f := New(true)
 
 	mux := http.NewServeMux()
 	mux.HandleFunc("/ws", func(w http.ResponseWriter, req *http.Request) {
-		w.WriteHeader(400)
+		w.WriteHeader(http.StatusBadRequest)
 	})
-	srv := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
-		mux.ServeHTTP(w, req)
-	})
-	defer srv.Close()
+
+	srv := httptest.NewServer(mux)
+	t.Cleanup(srv.Close)
 
 	proxy := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
 		path := req.URL.Path // keep the original path
@@ -565,14 +426,10 @@ func TestWebSocketUpgradeFailed(t *testing.T) {
 			// Set new backend URL
 			req.URL = testutils.ParseURI(srv.URL)
 			req.URL.Path = path
-			websocketRequest := IsWebsocketRequest(req)
-			assert.Equal(t, true, websocketRequest)
 			f.ServeHTTP(w, req)
-		} else {
-			w.WriteHeader(200)
 		}
 	})
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	proxyAddr := proxy.Listener.Addr().String()
 	conn, err := net.DialTimeout("tcp", proxyAddr, dialTimeout)
@@ -586,7 +443,7 @@ func TestWebSocketUpgradeFailed(t *testing.T) {
 	req.Header.Add("upgrade", "websocket")
 	req.Header.Add("Connection", "upgrade")
 
-	req.Write(conn)
+	_ = req.Write(conn)
 
 	// First request works with 400
 	br := bufio.NewReader(conn)
@@ -597,21 +454,18 @@ func TestWebSocketUpgradeFailed(t *testing.T) {
 }
 
 func TestForwardsWebsocketTraffic(t *testing.T) {
-	f, err := New()
-	require.NoError(t, err)
+	f := New(true)
 
 	mux := http.NewServeMux()
 	mux.Handle("/ws", websocket.Handler(func(conn *websocket.Conn) {
-		conn.Write([]byte("ok"))
-		conn.Close()
+		_, _ = conn.Write([]byte("ok"))
+		_ = conn.Close()
 	}))
-	srv := testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
-		mux.ServeHTTP(w, req)
-	})
-	defer srv.Close()
+	srv := httptest.NewServer(mux)
+	t.Cleanup(srv.Close)
 
 	proxy := createProxyWithForwarder(f, srv.URL)
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	proxyAddr := proxy.Listener.Addr().String()
 	resp, err := newWebsocketRequest(
@@ -646,11 +500,11 @@ func createTLSWebsocketServer() *httptest.Server {
 	return srv
 }
 
-func createProxyWithForwarder(forwarder *Forwarder, URL string) *httptest.Server {
+func createProxyWithForwarder(forwarder http.Handler, uri string) *httptest.Server {
 	return testutils.NewHandler(func(w http.ResponseWriter, req *http.Request) {
 		path := req.URL.Path // keep the original path
 		// Set new backend URL
-		req.URL = testutils.ParseURI(URL)
+		req.URL = testutils.ParseURI(uri)
 		req.URL.Path = path
 
 		forwarder.ServeHTTP(w, req)
@@ -659,17 +513,16 @@ func createProxyWithForwarder(forwarder *Forwarder, URL string) *httptest.Server
 
 func TestWebSocketTransferTLSConfig(t *testing.T) {
 	srv := createTLSWebsocketServer()
-	defer srv.Close()
+	t.Cleanup(srv.Close)
 
-	forwarderWithoutTLSConfig, err := New(PassHostHeader(true))
-	require.NoError(t, err)
+	forwarderWithoutTLSConfig := New(true)
 
 	proxyWithoutTLSConfig := createProxyWithForwarder(forwarderWithoutTLSConfig, srv.URL)
 	defer proxyWithoutTLSConfig.Close()
 
 	proxyAddr := proxyWithoutTLSConfig.Listener.Addr().String()
 
-	_, err = newWebsocketRequest(
+	_, err := newWebsocketRequest(
 		withServer(proxyAddr),
 		withPath("/ws"),
 		withData("ok"),
@@ -680,8 +533,8 @@ func TestWebSocketTransferTLSConfig(t *testing.T) {
 	transport := &http.Transport{
 		TLSClientConfig: &tls.Config{InsecureSkipVerify: true},
 	}
-	forwarderWithTLSConfig, err := New(PassHostHeader(true), RoundTripper(transport))
-	require.NoError(t, err)
+	forwarderWithTLSConfig := New(true)
+	forwarderWithTLSConfig.Transport = transport
 
 	proxyWithTLSConfig := createProxyWithForwarder(forwarderWithTLSConfig, srv.URL)
 	defer proxyWithTLSConfig.Close()
@@ -699,8 +552,7 @@ func TestWebSocketTransferTLSConfig(t *testing.T) {
 
 	http.DefaultTransport.(*http.Transport).TLSClientConfig = &tls.Config{InsecureSkipVerify: true}
 
-	forwarderWithTLSConfigFromDefaultTransport, err := New(PassHostHeader(true))
-	require.NoError(t, err)
+	forwarderWithTLSConfigFromDefaultTransport := New(true)
 
 	proxyWithTLSConfigFromDefaultTransport := createProxyWithForwarder(forwarderWithTLSConfigFromDefaultTransport, srv.URL)
 	defer proxyWithTLSConfig.Close()
@@ -717,7 +569,7 @@ func TestWebSocketTransferTLSConfig(t *testing.T) {
 	assert.Equal(t, "ok", resp)
 }
 
-const dialTimeout = time.Second
+const dialTimeout = clock.Second
 
 type websocketRequestOpt func(w *websocketRequest)
 
@@ -776,7 +628,8 @@ func (w *websocketRequest) send() (string, error) {
 	if _, err := conn.Write([]byte(w.Data)); err != nil {
 		return "", err
 	}
-	var msg = make([]byte, 512)
+
+	msg := make([]byte, 512)
 	var n int
 	n, err = conn.Read(msg)
 	if err != nil {
