@@ -2,30 +2,27 @@ package memmetrics
 
 import (
 	"testing"
-	"time"
 
-	"github.com/mailgun/timetools"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+	"github.com/vulcand/oxy/v2/internal/holsterv4/clock"
 )
 
 func TestCloneExpired(t *testing.T) {
-	clockTest := &timetools.FreezedTime{
-		CurrentTime: time.Date(2012, 3, 4, 5, 6, 7, 0, time.UTC),
-	}
+	clock.Freeze(clock.Date(2012, 3, 4, 5, 6, 7, 0, clock.UTC))
 
-	cnt, err := NewCounter(3, time.Second, CounterClock(clockTest))
+	cnt, err := NewCounter(3, clock.Second)
 	require.NoError(t, err)
 
 	cnt.Inc(1)
 
-	clockTest.Sleep(time.Second)
+	clock.Advance(clock.Second)
 	cnt.Inc(1)
 
-	clockTest.Sleep(time.Second)
+	clock.Advance(clock.Second)
 	cnt.Inc(1)
 
-	clockTest.Sleep(time.Second)
+	clock.Advance(clock.Second)
 	out := cnt.Clone()
 
 	assert.EqualValues(t, 2, out.Count())
