@@ -2,18 +2,18 @@ package roundrobin
 
 import (
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net/http"
 	"net/http/httptest"
 	"net/url"
 	"testing"
-	"time"
 
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
-	"github.com/vulcand/oxy/forward"
-	"github.com/vulcand/oxy/roundrobin/stickycookie"
-	"github.com/vulcand/oxy/testutils"
+	"github.com/vulcand/oxy/v2/forward"
+	"github.com/vulcand/oxy/v2/internal/holsterv4/clock"
+	"github.com/vulcand/oxy/v2/roundrobin/stickycookie"
+	"github.com/vulcand/oxy/v2/testutils"
 )
 
 func TestBasic(t *testing.T) {
@@ -23,8 +23,7 @@ func TestBasic(t *testing.T) {
 	defer a.Close()
 	defer b.Close()
 
-	fwd, err := forward.New()
-	require.NoError(t, err)
+	fwd := forward.New(false)
 
 	sticky := NewStickySession("test")
 	require.NotNil(t, sticky)
@@ -38,7 +37,7 @@ func TestBasic(t *testing.T) {
 	require.NoError(t, err)
 
 	proxy := httptest.NewServer(lb)
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	client := http.DefaultClient
 
@@ -51,7 +50,7 @@ func TestBasic(t *testing.T) {
 		require.NoError(t, err)
 
 		defer resp.Body.Close()
-		body, err := ioutil.ReadAll(resp.Body)
+		body, err := io.ReadAll(resp.Body)
 
 		require.NoError(t, err)
 		assert.Equal(t, "a", string(body))
@@ -65,8 +64,7 @@ func TestBasicWithHashValue(t *testing.T) {
 	defer a.Close()
 	defer b.Close()
 
-	fwd, err := forward.New()
-	require.NoError(t, err)
+	fwd := forward.New(false)
 
 	sticky := NewStickySession("test")
 	require.NotNil(t, sticky)
@@ -83,7 +81,7 @@ func TestBasicWithHashValue(t *testing.T) {
 	require.NoError(t, err)
 
 	proxy := httptest.NewServer(lb)
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	client := http.DefaultClient
 	var cookie *http.Cookie
@@ -97,7 +95,7 @@ func TestBasicWithHashValue(t *testing.T) {
 		resp, err := client.Do(req)
 		require.NoError(t, err)
 
-		body, err := ioutil.ReadAll(resp.Body)
+		body, err := io.ReadAll(resp.Body)
 		defer resp.Body.Close()
 
 		require.NoError(t, err)
@@ -119,13 +117,12 @@ func TestBasicWithAESValue(t *testing.T) {
 	defer a.Close()
 	defer b.Close()
 
-	fwd, err := forward.New()
-	require.NoError(t, err)
+	fwd := forward.New(false)
 
 	sticky := NewStickySession("test")
 	require.NotNil(t, sticky)
 
-	aesValue, err := stickycookie.NewAESValue([]byte("95Bx9JkKX3xbd7z3"), 5*time.Second)
+	aesValue, err := stickycookie.NewAESValue([]byte("95Bx9JkKX3xbd7z3"), 5*clock.Second)
 	require.NoError(t, err)
 
 	sticky.SetCookieValue(aesValue)
@@ -140,7 +137,7 @@ func TestBasicWithAESValue(t *testing.T) {
 	require.NoError(t, err)
 
 	proxy := httptest.NewServer(lb)
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	client := http.DefaultClient
 	var cookie *http.Cookie
@@ -154,7 +151,7 @@ func TestBasicWithAESValue(t *testing.T) {
 		resp, err := client.Do(req)
 		require.NoError(t, err)
 
-		body, err := ioutil.ReadAll(resp.Body)
+		body, err := io.ReadAll(resp.Body)
 		defer resp.Body.Close()
 
 		require.NoError(t, err)
@@ -175,8 +172,7 @@ func TestStickyCookie(t *testing.T) {
 	defer a.Close()
 	defer b.Close()
 
-	fwd, err := forward.New()
-	require.NoError(t, err)
+	fwd := forward.New(false)
 
 	sticky := NewStickySession("test")
 	require.NotNil(t, sticky)
@@ -190,7 +186,7 @@ func TestStickyCookie(t *testing.T) {
 	require.NoError(t, err)
 
 	proxy := httptest.NewServer(lb)
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	resp, err := http.Get(proxy.URL)
 	require.NoError(t, err)
@@ -284,13 +280,13 @@ func TestStickyCookieWithOptions(t *testing.T) {
 			desc: "Expires",
 			name: "test",
 			options: CookieOptions{
-				Expires: time.Date(1955, 11, 12, 1, 22, 0, 0, time.UTC),
+				Expires: clock.Date(1955, 11, 12, 1, 22, 0, 0, clock.UTC),
 			},
 			expected: &http.Cookie{
 				Name:       "test",
 				Value:      a.URL,
 				Path:       "/",
-				Expires:    time.Date(1955, 11, 12, 1, 22, 0, 0, time.UTC),
+				Expires:    clock.Date(1955, 11, 12, 1, 22, 0, 0, clock.UTC),
 				RawExpires: "Sat, 12 Nov 1955 01:22:00 GMT",
 				Raw:        fmt.Sprintf("test=%s; Path=/; Expires=Sat, 12 Nov 1955 01:22:00 GMT", a.URL),
 			},
@@ -327,9 +323,7 @@ func TestStickyCookieWithOptions(t *testing.T) {
 
 	for _, test := range testCases {
 		t.Run(test.desc, func(t *testing.T) {
-
-			fwd, err := forward.New()
-			require.NoError(t, err)
+			fwd := forward.New(false)
 
 			sticky := NewStickySessionWithOptions(test.name, test.options)
 			require.NotNil(t, sticky)
@@ -343,7 +337,7 @@ func TestStickyCookieWithOptions(t *testing.T) {
 			require.NoError(t, err)
 
 			proxy := httptest.NewServer(lb)
-			defer proxy.Close()
+			t.Cleanup(proxy.Close)
 
 			resp, err := http.Get(proxy.URL)
 			require.NoError(t, err)
@@ -361,8 +355,7 @@ func TestRemoveRespondingServer(t *testing.T) {
 	defer a.Close()
 	defer b.Close()
 
-	fwd, err := forward.New()
-	require.NoError(t, err)
+	fwd := forward.New(false)
 
 	sticky := NewStickySession("test")
 	require.NotNil(t, sticky)
@@ -376,7 +369,7 @@ func TestRemoveRespondingServer(t *testing.T) {
 	require.NoError(t, err)
 
 	proxy := httptest.NewServer(lb)
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	client := http.DefaultClient
 
@@ -390,7 +383,7 @@ func TestRemoveRespondingServer(t *testing.T) {
 		require.NoError(t, errReq)
 		defer resp.Body.Close()
 
-		body, errReq := ioutil.ReadAll(resp.Body)
+		body, errReq := io.ReadAll(resp.Body)
 		require.NoError(t, errReq)
 
 		assert.Equal(t, "a", string(body))
@@ -417,7 +410,7 @@ func TestRemoveRespondingServer(t *testing.T) {
 		require.NoError(t, err)
 
 		defer resp.Body.Close()
-		body, err := ioutil.ReadAll(resp.Body)
+		body, err := io.ReadAll(resp.Body)
 
 		require.NoError(t, err)
 		assert.Equal(t, "b", string(body))
@@ -431,8 +424,7 @@ func TestRemoveAllServers(t *testing.T) {
 	defer a.Close()
 	defer b.Close()
 
-	fwd, err := forward.New()
-	require.NoError(t, err)
+	fwd := forward.New(false)
 
 	sticky := NewStickySession("test")
 	require.NotNil(t, sticky)
@@ -446,7 +438,7 @@ func TestRemoveAllServers(t *testing.T) {
 	require.NoError(t, err)
 
 	proxy := httptest.NewServer(lb)
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	client := http.DefaultClient
 
@@ -459,7 +451,7 @@ func TestRemoveAllServers(t *testing.T) {
 		require.NoError(t, errReq)
 		defer resp.Body.Close()
 
-		body, errReq := ioutil.ReadAll(resp.Body)
+		body, errReq := io.ReadAll(resp.Body)
 		require.NoError(t, errReq)
 
 		assert.Equal(t, "a", string(body))
@@ -484,8 +476,7 @@ func TestBadCookieVal(t *testing.T) {
 
 	defer a.Close()
 
-	fwd, err := forward.New()
-	require.NoError(t, err)
+	fwd := forward.New(false)
 
 	sticky := NewStickySession("test")
 	require.NotNil(t, sticky)
@@ -497,7 +488,7 @@ func TestBadCookieVal(t *testing.T) {
 	require.NoError(t, err)
 
 	proxy := httptest.NewServer(lb)
-	defer proxy.Close()
+	t.Cleanup(proxy.Close)
 
 	client := http.DefaultClient
 
@@ -508,7 +499,7 @@ func TestBadCookieVal(t *testing.T) {
 	resp, err := client.Do(req)
 	require.NoError(t, err)
 
-	body, err := ioutil.ReadAll(resp.Body)
+	body, err := io.ReadAll(resp.Body)
 	require.NoError(t, err)
 	assert.Equal(t, "a", string(body))
 
@@ -519,7 +510,7 @@ func TestBadCookieVal(t *testing.T) {
 	resp, err = client.Do(req)
 	require.NoError(t, err)
 
-	_, err = ioutil.ReadAll(resp.Body)
+	_, err = io.ReadAll(resp.Body)
 	require.NoError(t, err)
 	assert.Equal(t, http.StatusInternalServerError, resp.StatusCode)
 }
@@ -537,10 +528,11 @@ func TestStickySession_GetBackend(t *testing.T) {
 	rawValue := &stickycookie.RawValue{}
 	hashValue := &stickycookie.HashValue{}
 	saltyHashValue := &stickycookie.HashValue{Salt: "test salt"}
-	aesValue, err := stickycookie.NewAESValue([]byte("95Bx9JkKX3xbd7z3"), 5*time.Second)
+	aesValue, err := stickycookie.NewAESValue([]byte("95Bx9JkKX3xbd7z3"), 5*clock.Second)
+	require.NoError(t, err)
 	aesValueInfinite, err := stickycookie.NewAESValue([]byte("95Bx9JkKX3xbd7z3"), 0)
 	require.NoError(t, err)
-	aesValueExpired, err := stickycookie.NewAESValue([]byte("95Bx9JkKX3xbd7z3"), 1*time.Nanosecond)
+	aesValueExpired, err := stickycookie.NewAESValue([]byte("95Bx9JkKX3xbd7z3"), 1*clock.Nanosecond)
 	require.NoError(t, err)
 
 	tests := []struct {
