@@ -9,8 +9,10 @@ import (
 	"net/url"
 )
 
-// SerializableHttpRequest serializable HTTP request
-type SerializableHttpRequest struct {
+// SerializableHTTPRequest serializable HTTP request.
+//
+//nolint:musttag // Cannot be changed more now.
+type SerializableHTTPRequest struct {
 	Method           string
 	URL              *url.URL
 	Proto            string // "HTTP/1.0"
@@ -29,13 +31,13 @@ type SerializableHttpRequest struct {
 	TLS              *tls.ConnectionState
 }
 
-// Clone clone a request
-func Clone(r *http.Request) *SerializableHttpRequest {
+// Clone clone a request.
+func Clone(r *http.Request) *SerializableHTTPRequest {
 	if r == nil {
 		return nil
 	}
 
-	rc := new(SerializableHttpRequest)
+	rc := new(SerializableHTTPRequest)
 	rc.Method = r.Method
 	rc.URL = r.URL
 	rc.Proto = r.Proto
@@ -49,16 +51,16 @@ func Clone(r *http.Request) *SerializableHttpRequest {
 	return rc
 }
 
-// ToJson serializes to JSON
-func (s *SerializableHttpRequest) ToJson() string {
+// ToJSON serializes to JSON.
+func (s *SerializableHTTPRequest) ToJSON() string {
 	jsonVal, err := json.Marshal(s)
 	if err != nil || jsonVal == nil {
-		return fmt.Sprintf("Error marshalling SerializableHttpRequest to json: %s", err)
+		return fmt.Sprintf("error marshaling SerializableHTTPRequest to json: %s", err)
 	}
 	return string(jsonVal)
 }
 
-// DumpHttpRequest dump a HTTP request to JSON
-func DumpHttpRequest(req *http.Request) string {
-	return Clone(req).ToJson()
+// DumpHTTPRequest dump a HTTP request to JSON.
+func DumpHTTPRequest(req *http.Request) string {
+	return Clone(req).ToJSON()
 }
