@@ -10,7 +10,7 @@ import (
 
 type readCloserTestImpl struct{}
 
-func (r *readCloserTestImpl) Read(p []byte) (n int, err error) {
+func (r *readCloserTestImpl) Read(_ []byte) (n int, err error) {
 	return 0, nil
 }
 
@@ -19,7 +19,7 @@ func (r *readCloserTestImpl) Close() error {
 }
 
 // Just to make sure we don't panic, return err and not
-// username and pass and cover the function
+// username and pass and cover the function.
 func TestHttpReqToString(t *testing.T) {
 	req := &http.Request{
 		URL:    &url.URL{Host: "localhost:2374", Path: "/unittest"},
@@ -28,5 +28,5 @@ func TestHttpReqToString(t *testing.T) {
 		Body:   &readCloserTestImpl{},
 	}
 
-	assert.True(t, len(DumpHttpRequest(req)) > 0)
+	assert.True(t, len(DumpHTTPRequest(req)) > 0)
 }
